#ifndef COMMON_GENERATE_LIBRARY_DEPENDENCIES_LA_OBJECT_HPP
#define COMMON_GENERATE_LIBRARY_DEPENDENCIES_LA_OBJECT_HPP

/*
 * This file is part of the Score-P software ecosystem (http://www.score-p.org)
 *
 * Copyright (c) 2009-2013,
 * RWTH Aachen University, Germany
 *
 * Copyright (c) 2009-2013,
 * Gesellschaft fuer numerische Simulation mbH Braunschweig, Germany
 *
 * Copyright (c) 2009-2014,
 * Technische Universitaet Dresden, Germany
 *
 * Copyright (c) 2009-2013,
 * University of Oregon, Eugene, USA
 *
 * Copyright (c) 2009-2013, 2017, 2023,
 * Forschungszentrum Juelich GmbH, Germany
 *
 * Copyright (c) 2009-2013,
 * German Research School for Simulation Sciences GmbH, Juelich/Aachen, Germany
 *
 * Copyright (c) 2009-2013,
 * Technische Universitaet Muenchen, Germany
 *
 * This software may be modified and distributed under the terms of
 * a BSD-style license.  See the COPYING file in the package base
 * directory for details.
 *
 */


/**
 * @file
 */

#include <string>
#include <deque>
#include <sstream>
#include <algorithm>

/**
 * Helper struct to represent the content of one .la file. Used in code
 * generated by generate-library-dependencies.sh. To make usage easier
 * everything is public.
 */
struct la_object
{
    /**
     * Empty constructor. Needed to allow copies of STL containers containing this
     * class
     */
    la_object( void )
    {
    }

    /**
     * Copy constructor.
     */
    la_object( const la_object& source )
        :
        m_lib_name( source.m_lib_name ),
        m_build_dir( source.m_build_dir ),
        m_install_dir( source.m_install_dir ),
        m_libs( source.m_libs ),
        m_ldflags( source.m_ldflags ),
        m_rpath( source.m_rpath ),
        m_dependency_las( source.m_dependency_las )
    {
    }


    /**
     * Regular constructor.
     */
    la_object( const std::string& lib_name,
               const std::string& build_dir,
               const std::string& install_dir,
               const std::string& libs,
               const std::string& libdirs )
        :
        m_lib_name( lib_name ),
        m_build_dir( build_dir ),
        m_install_dir( install_dir ),
        m_libs( gen_libs( libs ) ),
        m_ldflags( gen_ldflags( libdirs ) ),
        m_rpath( gen_rpaths( libdirs ) )
    {
    }

    /**
     * Destructor.
     */
    virtual
    ~la_object()
    {
    }

    /**
     * Member.
     */
    std::string             m_lib_name;
    std::string             m_build_dir;
    std::string             m_install_dir;
    std::deque<std::string> m_libs;           // -l<lib>
    std::deque<std::string> m_ldflags;        // -L<dir>
    std::deque<std::string> m_rpath;          // -R<dir>
    std::deque<std::string> m_dependency_las; // unused

    // Helper methods to generate la_object members from strings in
    // <builddir>/src/config-external-libs-@AFS_PACKAGE_BUILD_name@.h.

    static std::deque<std::string>
    gen_deque( std::stringstream ss, std::string prefix )
    {
        std::deque<std::string> output;
        std::string             item;
        while ( ss >> item )
        {
            output.push_back( prefix + item );
        }
        return output;
    }

    static std::deque<std::string>
    gen_libs( std::string input )
    {
        return gen_deque( std::stringstream( input ), "" );
    }

    static std::deque<std::string>
    gen_ldflags( std::string input )
    {
        std::replace( input.begin(), input.end(), ':', ' ' );
        return gen_deque( std::stringstream( input ), "-L" );
    }

    static std::deque<std::string>
    gen_rpaths( std::string input )
    {
        std::replace( input.begin(), input.end(), ':', ' ' );
        return gen_deque( std::stringstream( input ), "-R" );
    }
};

#endif /* COMMON_GENERATE_LIBRARY_DEPENDENCIES_LA_OBJECT_HPP */
